Index: include/speex/speex_preprocess.h
===================================================================
--- include/speex/speex_preprocess.h	(revision 10303)
+++ include/speex/speex_preprocess.h	(working copy)
@@ -59,7 +59,9 @@
    float  reverb_level;
    float  speech_prob_start;
    float  speech_prob_continue;
-   
+   int adaptation_time;
+   int consec_noise_duration; /**< Number of consecutive noise frames required before considered speech*/
+
    float *frame;             /**< Processing frame (2*ps_size) */
    float *ps;                /**< Current power spectrum */
    float *gain2;             /**< Adjusted gains */
@@ -121,8 +123,9 @@
 /** Used like the ioctl function to control the preprocessor parameters */
 int speex_preprocess_ctl(SpeexPreprocessState *st, int request, void *ptr);
 
+/** Resets a denoising state */
+void speex_preprocess_state_reset(SpeexPreprocessState *st);
 
-
 /** Set preprocessor denoiser state */
 #define SPEEX_PREPROCESS_SET_DENOISE 0
 /** Get preprocessor denoiser state */
@@ -164,6 +167,12 @@
 #define SPEEX_PREPROCESS_SET_PROB_CONTINUE 16
 #define SPEEX_PREPROCESS_GET_PROB_CONTINUE 17
 
+#define SPEEX_PREPROCESS_SET_ADAPTATION_TIME 18
+#define SPEEX_PREPROCESS_GET_ADAPTATION_TIME 19
+
+#define SPEEX_PREPROCESS_SET_MIN_NOISE_DURATION 20
+#define SPEEX_PREPROCESS_GET_MIN_NOISE_DURATION 21
+
 #ifdef __cplusplus
 }
 #endif
Index: libspeex/preprocess.c
===================================================================
--- libspeex/preprocess.c	(revision 10303)
+++ libspeex/preprocess.c	(working copy)
@@ -59,6 +59,7 @@
 #define ZMAX .316
 #define ZMIN_1 10
 #define LOG_MIN_MAX_1 0.86859
+#define NOISE_OVERCOMPENS 1.4
 
 static void conj_window(float *w, int len)
 {
@@ -69,15 +70,19 @@
       int inv=0;
       if (x<1)
       {
-      } else if (x<2)
+      }
+	  else if (x<2)
       {
          x=2-x;
          inv=1;
-      } else if (x<3)
+      }
+	  else if (x<3)
       {
          x=x-2;
          inv=1;
-      } else {
+      }
+	  else
+	  {
          x=4-x;
       }
       x*=1.9979;
@@ -119,15 +124,15 @@
 
 SpeexPreprocessState *speex_preprocess_state_init(int frame_size, int sampling_rate)
 {
-   int i;
    int N, N3, N4;
 
    SpeexPreprocessState *st = (SpeexPreprocessState *)speex_alloc(sizeof(SpeexPreprocessState));
    st->frame_size = frame_size;
+   st->ps_size = st->frame_size;
 
    /* Round ps_size down to the nearest power of two */
 #if 0
-   i=1;
+   int i=1;
    st->ps_size = st->frame_size;
    while(1)
    {
@@ -143,8 +148,6 @@
    
    if (st->ps_size < 3*st->frame_size/4)
       st->ps_size = st->ps_size * 3 / 2;
-#else
-   st->ps_size = st->frame_size;
 #endif
 
    N = st->ps_size;
@@ -157,11 +160,13 @@
    st->agc_level = 8000;
    st->vad_enabled = 0;
    st->dereverb_enabled = 0;
-   st->reverb_decay = .5;
-   st->reverb_level = .2;
+   st->reverb_decay = .5f;
+   st->reverb_level = .2f;
 
    st->speech_prob_start = SPEEX_PROB_START_DEFAULT;
    st->speech_prob_continue = SPEEX_PROB_CONTINUE_DEFAULT;
+   st->adaptation_time = 200;
+   st->consec_noise_duration = 3;
 
    st->frame = (float*)speex_alloc(2*N*sizeof(float));
    st->ps = (float*)speex_alloc(N*sizeof(float));
@@ -182,7 +187,6 @@
    st->Smin = (float*)speex_alloc(N*sizeof(float));
    st->Stmp = (float*)speex_alloc(N*sizeof(float));
    st->update_prob = (float*)speex_alloc(N*sizeof(float));
-
    st->zeta = (float*)speex_alloc(N*sizeof(float));
    st->Zpeak = 0;
    st->Zlast = 0;
@@ -193,10 +197,25 @@
    st->speech_bands2 = (float*)speex_alloc(NB_BANDS*sizeof(float));
    st->noise_bandsN = st->speech_bandsN = 1;
 
-   conj_window(st->window, 2*N3);
+   conj_window(st->window, 2*N3);
+   speex_preprocess_state_reset(st);
+
+   st->fft_lookup = (struct drft_lookup*)speex_alloc(sizeof(struct drft_lookup));
+   spx_drft_init(st->fft_lookup,2*N);
+
+   return st;
+}
+
+void speex_preprocess_state_reset(SpeexPreprocessState* st)
+{
+   int N, N3, N4, i;
+   N = st->ps_size;
+   N3 = 2*N - st->frame_size;
+   N4 = st->frame_size - N3;
+
    for (i=2*N3;i<2*st->ps_size;i++)
       st->window[i]=1;
-   
+
    if (N4>0)
    {
       for (i=N3-1;i>=0;i--)
@@ -223,7 +242,7 @@
 
    for (i=0;i<N;i++)
    {
-      float ff=((float)i)*.5*sampling_rate/((float)N);
+      float ff=((float)i)*.5*st->sampling_rate/((float)N);
       st->loudness_weight[i] = .35f-.35f*ff/16000.f+.73f*exp(-.5f*(ff-3800)*(ff-3800)/9e5f);
       if (st->loudness_weight[i]<.01f)
          st->loudness_weight[i]=.01f;
@@ -235,14 +254,9 @@
    st->loudness = pow(6000,LOUDNESS_EXP);
    st->loudness2 = 6000;
    st->nb_loudness_adapt = 0;
-
-   st->fft_lookup = (struct drft_lookup*)speex_alloc(sizeof(struct drft_lookup));
-   spx_drft_init(st->fft_lookup,2*N);
-
    st->nb_adapt=0;
    st->consec_noise=0;
-   st->nb_preprocess=0;
-   return st;
+   st->nb_preprocess=0;
 }
 
 void speex_preprocess_state_destroy(SpeexPreprocessState *st)
@@ -293,13 +307,11 @@
    {
       for (i=0;i<st->ps_size;i++)
          st->noise[i] = (1.f-beta)*st->noise[i] + beta*ps[i];
-   } else {
+   }
+   else
+   {
       for (i=0;i<st->ps_size;i++)
          st->noise[i] = (1.f-beta)*st->noise[i] + beta*max(1.f,ps[i]-echo[i]); 
-#if 0
-      for (i=0;i<st->ps_size;i++)
-         st->noise[i] = 0;
-#endif
    }
 }
 
@@ -351,7 +363,7 @@
 
       if (st->noise_bandsN < 50 || st->speech_bandsN < 50)
       {
-         if (mean_post > 5.f)
+         if (mean_post > 5.0f)
          {
             float adapt = 1./st->speech_bandsN++;
             if (adapt<.005f)
@@ -362,7 +374,9 @@
                /*st->speech_bands2[i] = (1-adapt)*st->speech_bands2[i] + adapt*bands[i]*bands[i];*/
                st->speech_bands2[i] = (1.f-adapt)*st->speech_bands2[i] + adapt*(bands[i]-st->speech_bands[i])*(bands[i]-st->speech_bands[i]);
             }
-         } else {
+         }
+		 else
+		 {
             float adapt = 1./st->noise_bandsN++;
             if (adapt<.005f)
                adapt = .005f;
@@ -605,14 +619,16 @@
          st->Smin[i] = st->Stmp[i] = st->S[i]+100.f;
    }
 
-   if (st->nb_preprocess%200==0)
+   if (st->nb_preprocess%st->adaptation_time==0)
    {
       for (i=1;i<N-1;i++)
       {
          st->Smin[i] = min(st->Stmp[i], st->S[i]);
          st->Stmp[i] = st->S[i];
       }
-   } else {
+   }
+   else
+   {
       for (i=1;i<N-1;i++)
       {
          st->Smin[i] = min(st->Smin[i], st->S[i]);
@@ -627,11 +643,8 @@
       /*fprintf (stderr, "%f ", st->S[i]/st->Smin[i]);*/
       /*fprintf (stderr, "%f ", st->update_prob[i]);*/
    }
-
 }
 
-#define NOISE_OVERCOMPENS 1.4
-
 int speex_preprocess(SpeexPreprocessState *st, spx_int16_t *x, float *echo)
 {
    int i;
@@ -643,7 +656,7 @@
    int N4 = st->frame_size - N3;
    float scale=.5f/N;
    float *ps=st->ps;
-   float Zframe=0, Pframe;
+   float Zframe=0.0f, Pframe = 0.0f;
 
    preprocess_analysis(st, x);
 
@@ -655,6 +668,10 @@
    if (st->nb_adapt<10)
    {
       update_noise(st, ps, echo);
+      /* Special case for first frame */
+      if (st->nb_adapt==1)
+         for (i=1;i<N;i++)
+            st->old_ps[i] = ps[i];
    }
 
    /* Deal with residual echo if provided */
@@ -677,11 +694,6 @@
    if (mean_post<0.f)
       mean_post=0.f;
 
-   /* Special case for first frame */
-   if (st->nb_adapt==1)
-      for (i=1;i<N;i++)
-         st->old_ps[i] = ps[i];
-
    /* Compute a priori SNR */
    {
       /* A priori update rate */
@@ -737,11 +749,12 @@
    if (st->vad_enabled)
       is_speech = speex_compute_vad(st, ps, mean_prior, mean_post);
 
-
-   if (st->consec_noise>=3)
+   if (st->consec_noise>=st->consec_noise_duration)
    {
       update_noise(st, st->old_ps, echo);
-   } else {
+   }
+   else
+   {
       for (i=1;i<N-1;i++)
       {
          if (st->update_prob[i]<.5f/* || st->ps[i] < st->noise[i]*/)
@@ -816,7 +829,9 @@
          st->gain2[i] = p*p*st->gain[i];
          /*st->gain2[i]=(p*sqrt(st->gain[i])+.05*(1-p))*(p*sqrt(st->gain[i])+.05*(1-p));*/
          /*st->gain2[i] = pow(st->gain[i], p) * pow(.2f,1.f-p);*/
-      } else {
+      }
+	  else
+	  {
          st->gain2[i]=1.f;
       }
    }
@@ -1018,6 +1033,22 @@
       (*(int*)ptr) = st->speech_prob_continue * 100;
       break;
 
+	case SPEEX_PREPROCESS_SET_MIN_NOISE_DURATION:
+		st->consec_noise_duration = (*(int*)ptr);
+		break;
+
+	case SPEEX_PREPROCESS_GET_MIN_NOISE_DURATION:
+		(*(int*)ptr) = st->consec_noise_duration;
+		break;
+
+	case SPEEX_PREPROCESS_SET_ADAPTATION_TIME:
+		st->adaptation_time = (*(int*)ptr);
+		break;
+
+	case SPEEX_PREPROCESS_GET_ADAPTATION_TIME:
+		(*(int*)ptr) = st->adaptation_time;
+		break;
+
       default:
       speex_warning_int("Unknown speex_preprocess_ctl request: ", request);
       return -1;